/* Copyright Inspirel Sp. z o.o. 2013-2015.
 * This file is part of YAMI4.
 *
 * YAMI4 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAMI4 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAMI4.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef YAMI4INDUSTRY_BROKER_CONFIG_H
#define YAMI4INDUSTRY_BROKER_CONFIG_H

/* general YAMI4 limits like max length of target name */
#include "../limits.h"

#include <stdint.h>

enum subscription_overflow_policy
{
    reject_message,
    drop_update,
    unsubscribe
};

struct broker_properties
{
    /* listener endpoint                                       */
    /* this value should be known to all communicating parties */
    char listener[MAX_LENGTH_OF_TARGET_NAME];

    /* overflow policy                                                      */
    /* possible values are:                                                 */
    /* reject_message - if the message was sent via "publish-confirm",      */
    /*                  it will be rejected                                 */
    /* drop_update    - the update will be skipped for the overflown client */
    /* unsubscribe    - the whole subscription will be dropped              */
    enum subscription_overflow_policy overflow_policy;

    /* initial logging levels                                            */
    /* log.main - main (startup) part of the broker with the worker task */
    /* log.subscriptions - actions on the routing table                  */
    /* log.messages - routing of individual messages (note: high volume) */
    int32_t log_main;
    int32_t log_subscriptions;
    int32_t log_messages;
};

void init_configuration(struct broker_properties * out_config);

#endif /* YAMI4INDUSTRY_BROKER_CONFIG_H */
