/* Copyright Inspirel Sp. z o.o. 2013-2015.
 * This file is part of YAMI4.
 *
 * YAMI4 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAMI4 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAMI4.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "patterns.h"
#include "broker_limits.h"
#include "../utils.h"

const int32_t patterns_comma = ',';
const int32_t patterns_dot = '.';
const int32_t patterns_star = '*';

/* helper for single string vs. single pattern matching */
static int32_t patterns_hierarchic_match(const char str[], const char pat[])
{
    int32_t result = -1; /* unknown */

    size_t si = 0u;
    size_t pi = 0u;
    
    while ((result == -1) &&
        ((int32_t)str[si] != (int32_t)'\0') &&
        ((int32_t)pat[pi] != (int32_t)'\0'))
    {
        if ((int32_t)pat[pi] == patterns_star)
        {
            /* skip characters in strings until dot or end */
            while (((int32_t)str[si] != (int32_t)'\0') &&
                ((int32_t)str[si] != patterns_dot))
            {
                ++si;
            }
            
            /* if star is the last character in the pattern, */
            /* then we consider the rest of strings to match */
            /* (this is the "hierarchic" part of matching)   */
            
            if ((int32_t)pat[pi + 1u] == (int32_t)'\0')
            {
                result = 1; /* true */
            }
            else
            {
                /* otherwise move on with the pattern, */
                /* there are more components to check  */
                ++pi;
            }
        }
        else if ((int32_t)str[si] != (int32_t)pat[pi])
        {
            result = 0; /* false */
        }
        else
        {
            ++si;
            ++pi;
        }
    }
    
    if (result == -1)
    {
        if (((int32_t)str[si] == (int32_t)'\0') &&
            ((int32_t)pat[pi] == (int32_t)'\0'))
        {
            result = 1; /* true */
        }
        else
        {
            result = 0; /* false */
        }
    }

    return result;
}

int32_t patterns_multi_hierarchic_match(
    const char strings[], const char patterns[])
{
    const size_t strings_len = yami_strlen(strings);
    const size_t patterns_len = yami_strlen(patterns);

    int32_t result = 0; /* false unless match is found */
    int32_t single_result;
    size_t i;
    size_t j;    

    char strings2[MAX_TAG_PATTERN_LENGTH];
    char patterns2[MAX_TAG_PATTERN_LENGTH];

    if (yami_strcmp(strings, patterns) == 0)
    {
        /* in the simple (and most frequent) case */
        /* there is only one tag and one pattern, so it can be optimized away */

        result = 1; /* true */
    }
    else
    {
        yami_strncpy(strings2, strings, strings_len);
        strings2[strings_len] = (char)'\0';

        yami_strncpy(patterns2, patterns, patterns_len);
        patterns2[patterns_len] = (char)'\0';

        /* divide strings and patterns into single tokens */
        /* by just injecting zeros in place of separators */
        for (i = 0u; i != strings_len; ++i)
        {
            if ((int32_t)strings2[i] == patterns_comma)
            {
                strings2[i] = (char)'\0';
            }
        }

        for (j = 0u; j != patterns_len; ++j)
        {
            if ((int32_t)patterns2[j] == patterns_comma)
            {
                patterns2[j] = (char)'\0';
            }
        }
    
        /* try to match every tag with every pattern        */
        /* - at least one match means the whole is accepted */
    
        j = 0u; /* index for patterns */
        while (j < patterns_len)
        {
            i = 0u; /* index for tags */
            while (i < strings_len)
            {
                single_result = patterns_hierarchic_match(&strings2[i], &patterns2[j]);
                if (single_result != 0)
                {
                    result = 1; /* true */

                    break;
                }
                else
                {
                    /* move to the next tag in strings */
                    while ((int32_t)strings2[i] != (int32_t)'\0')
                    {
                        ++i;
                    }

                    ++i;
                }
            }

            if (result != 0)
            {
                /* result is already positive (pattern matches) */

                break;
            }

            /* move to the next pattern */
            while ((int32_t)patterns2[j] != (int32_t)'\0')
            {
                ++j;
            }

            ++j;
        }
    }
    
    return result;
}
