/* Copyright Inspirel Sp. z o.o. 2013-2015.
 * This file is part of YAMI4.
 *
 * YAMI4 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAMI4 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAMI4.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef YAMI4INDUSTRY_BROKER_ROUTING_H
#define YAMI4INDUSTRY_BROKER_ROUTING_H

#include "../limits.h"

#include <stdint.h>
#include <stddef.h>


/**
 * Initializes internal subscription table.
 */
void routing_init(void);

/**
 * Inserts a new subscription for the given list of tags,
 * destination object and target, which is typically the same
 * as the source of subscribing message.
 * Returns 1 if the subscription was successful and 0 otherwise
 * (note: the only reason for failure is exhaustion of subscription table).
 */
int32_t routing_subscribe(
    const char tags[],
    const char destination_obj[],
    const char source[]);

/**
 * Returns 1 (true) if the list of tags matches subscription
 * at the given index, in which case the target and object name
 * are filled to point where the message should be forwarded.
 */
int32_t routing_check_matching_entry(
    const char tags[],
    size_t index,
    char out_target[], char out_object_name[]);

/**
 * Deactivates subscription at the given index.
 * Called when the given subscription overflows on the target channel
 * (that is, the target channel is still busy while there is a new
 * message already coming) and the overflow policy is set to unsubscribe.
 */
void routing_unsubscribe_at_index(size_t index);
        
void routing_message_progress(
    const char target[]);

#endif /* YAMI4INDUSTRY_BROKER_ROUTING_H */
