/* Copyright Inspirel Sp. z o.o. 2013-2015.
 * This file is part of YAMI4.
 *
 * YAMI4 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAMI4 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAMI4.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef YAMI4INDUSTRY_CALLBACKS_H
#define YAMI4INDUSTRY_CALLBACKS_H

#include <stddef.h>
#include <stdint.h>

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

struct yami_agent;

/** Type of function callback for raw message delivery.
 *
 * @param target endpoint name of the message sender
 * @param header header buffer
 * @param header_size size of the header buffer
 * @param body body buffer
 * @param body_size size of the body buffer
 */
typedef void (*yami_raw_message_function)(const char target[],
    const uint8_t header[], size_t header_size,
    const uint8_t body[], size_t body_size);

/** Type of message for high-level message delivery.
 */
enum yami_message_type
{
    yami_incoming_message,  /* new incoming message           */
    yami_message_reply,     /* reply from remote receiver     */
    yami_message_rejected,  /* outgoing message was rejected  */
    yami_message_abandoned  /* outgoing message was abandoned */
};

/** Type of function callback for high-level message delivery.
 *
 * @param target endpoint name of the message sender
 * @param message_type type of message notification
 * @param object_name name of logical message destination
 *                    (valid only for yami_incoming_message)
 * @param message_name name of message
 *                     (valid only for yami_incoming_message)
 * @param exception_msg description of problem at the receiver side
 *                      (valid only for yami_incoming_message)
 * @param message_id high-level message identifier
 * @param body body buffer
 *             (valid only for yami_incoming_message and for yami_message_reply)
 * @param body_size size of the body buffer
 *                  (valid only for yami_incoming_message and for yami_message_reply)
 */
typedef void (*yami_message_function)(const char target[],
    enum yami_message_type message_type,
    const char object_name[], const char message_name[], const char exception_msg[],
    int64_t message_id,
    const uint8_t body[], size_t body_size);

/** Type of function callback for I/O error logging.
 *
 * @param error_code copy of errno or other error indicator as set by the system
 *                   or 0 if exact error number cannot be obtained
 * @param description error message 
 */
typedef void (*yami_io_error_function)(int32_t error_code,
    const char description[]);

/** Type of connection event notification
 */
enum yami_connection_event
{
    yami_new_outgoing_connection,
    yami_new_incoming_connection,
    yami_connection_closed
};

/** Type of function callback for reporting connection events.
 *
 * @param event kind of connection event (open, closed)
 * @param target remote endpoint name
 */
typedef void (*yami_connection_event_function)(
    enum yami_connection_event event, const char target[]);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* YAMI4INDUSTRY_CALLBACKS_H */
