/* Copyright Inspirel Sp. z o.o. 2013-2015.
 * This file is part of YAMI4.
 *
 * YAMI4 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAMI4 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAMI4.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef YAMI4INDUSTRY_CHANNEL_H
#define YAMI4INDUSTRY_CHANNEL_H

#include "callbacks.h"
#include "common.h"
#include "limits.h"
#include "options.h"

#include <stddef.h>
#include <stdint.h>

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

enum yami_protocol { yami_tcp, yami_udp };

struct yami_channel
{
    int32_t socket_;

    enum yami_protocol protocol_;

    uint32_t host_;
    uint16_t port_;

    int32_t socket_ready_for_output_;
    int32_t socket_ready_for_input_;

    int32_t output_buffer_has_data_;
    size_t output_data_size_;
    size_t output_data_written_so_far_;
    int64_t output_message_id_; /* for high-level callbacks */
    
    int32_t input_buffer_has_space_;
    size_t input_data_read_so_far_;
    uint32_t input_message_id_;
    size_t input_message_header_size_;
    size_t input_message_payload_size_;
    int32_t input_message_complete_;

    /* note: sizes of the arrays below can interact */
    /* with required padding at the end of struct   */
    
    char target_[MAX_LENGTH_OF_TARGET_NAME];
    
    uint8_t output_buffer_[MAX_OUTGOING_MESSAGE_PAYLOAD];
    uint8_t input_buffer_[MAX_INCOMING_MESSAGE_PAYLOAD];
};

void yami_channel_init(struct yami_channel * self);

int32_t yami_channel_is_open(const struct yami_channel * self);

int32_t yami_channel_name_cmp(const struct yami_channel * self,
    const char target[]);

enum yami_result yami_channel_open(struct yami_channel * self,
    const char target[], const struct yami_options * options,
    yami_io_error_function io_error_callback);

enum yami_result yami_channel_close(struct yami_channel * self,
    yami_io_error_function io_error_callback);

enum yami_result yami_channel_post_raw_message(struct yami_channel * self,
    uint32_t core_message_id,
    const uint8_t header[], size_t header_size,
    const uint8_t body[], size_t body_size);

enum yami_result yami_channel_post_message(struct yami_channel * self,
    const char object_name[], const char message_name[],
    int64_t message_id, uint32_t core_message_id,
    const uint8_t body[], size_t body_size);

enum yami_result yami_channel_post_reply(struct yami_channel * self,
    int64_t message_id, uint32_t core_message_id,
    const uint8_t body[], size_t body_size);

enum yami_result yami_channel_post_reject(struct yami_channel * self,
    int64_t message_id, uint32_t core_message_id,
    const char reason[]);

enum yami_result yami_channel_write_some_data(struct yami_channel * self,
    yami_io_error_function io_error_callback);

enum yami_result yami_channel_read_some_data(struct yami_channel * self,
    yami_io_error_function io_error_callback);

void yami_channel_clean_input_buffer(struct yami_channel * const self);

enum yami_result yami_channel_check_udp_packet(
    struct yami_channel * self);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* YAMI4INDUSTRY_CHANNEL_H */
