/* Copyright Inspirel Sp. z o.o. 2013-2015.
 * This file is part of YAMI4.
 *
 * YAMI4 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAMI4 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAMI4.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef YAMI4INDUSTRY_COMMON_H
#define YAMI4INDUSTRY_COMMON_H

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

/**  General type for reporting success and error states.
 */
enum yami_result
{
    yami_ok,                /* < Operation completed successfully. */
    yami_no_such_name,      /* < The given name was not found. */
    yami_not_enough_space,  /* < There is not enough space in the buffer. */
    yami_unexpected_value,  /* < The given value was not recognized. */
    yami_bad_protocol,      /* < The connection protocol is incorrect. */
    yami_io_error,          /* < Unable to perform the I/O operation. */
    yami_timed_out,         /* < The requested operation timed out. */
    yami_channel_closed,    /* < The operation was not possible due to EOF. */
    yami_bad_state          /* < The given object is in the wrong state. */
};

/** Type of parameter entry.
 */
enum yami_parameter_type
{
    yami_boolean,                /*< bool */
    yami_integer,                /*< int */
    yami_long_long,              /*< long long */
#ifdef YAMI4_WITH_DOUBLE_FLOAT
    yami_double_float,           /*< double */
#endif /* YAMI4_WITH_DOUBLE_FLOAT */
    yami_string,                 /*< string */
    yami_binary,                 /*< Binary block */
    yami_boolean_array,          /*< Array of bool */
    yami_integer_array,          /*< Array of int */
    yami_long_long_array,        /*< Array of long long */
#ifdef YAMI4_WITH_DOUBLE_FLOAT
    yami_double_float_array,     /*< Array of double */
#endif /* YAMI4_WITH_DOUBLE_FLOAT */
    yami_string_array,           /*< Array of strings */
    yami_binary_array,           /*< Array of binary blocks */
    yami_nested_parameters,      /*< Nested parameters object */
    yami_nested_parameters_array /*< Array of nested parameters object */
};

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* YAMI4INDUSTRY_COMMON_H */
