/* Copyright Inspirel Sp. z o.o. 2013-2015.
 * This file is part of YAMI4.
 *
 * YAMI4 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAMI4 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAMI4.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "../../agent.h"
#include "../../serialization.h"

#include <assert.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

void message_callback(const char target[],
    enum yami_message_type message_type,
    const char object_name[], const char message_name[], const char exception_msg[],
    int64_t message_id,
    const uint8_t body[], size_t body_size)
{
    /* ignore the header, but deserialize the message body */

    enum yami_result res;
    
    int32_t num_of_entries;
    int32_t len;
    const char content_tag[] = "content";
    const size_t content_tag_len = 7; /* length of "content" */
    char field_name[8];               /* sizeof "content"    */
    char line[1000];
    enum yami_parameter_type type;
    
    /* buffer position is updated with each reading operation */
    size_t buffer_position = 0u;
    
    /* for simplicity this server does not allow unknown fields, */
    /* so the only expected message format is a regular message  */
    /* with "content" string field                               */
    
    res = yami_get_integer(body, body_size, &buffer_position, &num_of_entries);
    assert(res == yami_ok);
    
    if (num_of_entries != 1)
    {
        return;
    }
    
    /* read length of field name */
    
    res = yami_get_integer(body, body_size, &buffer_position, &len);
    assert(res == yami_ok);
    
    if (len != content_tag_len)
    {
        return;
    }
    
    /* read the field name (expect "content") */
    
    res = yami_get_raw_cstring(body, body_size, &buffer_position,
        field_name, (size_t)len);
    assert(res == yami_ok);
    
    if (strcmp(field_name, content_tag) != 0)
    {
        return;
    }
    
    /* read type (expect string) */
    
    res = yami_get_type(body, body_size, &buffer_position, &type);
    assert(res == yami_ok);
    
    if (type != yami_string)
    {
        return;
    }
    
    /* read length of field string value */
    
    res = yami_get_integer(body, body_size, &buffer_position, &len);
    assert(res == yami_ok);
    
    if (len >= 1000)
    {
        return;
    }
    
    /* read the field string value, and print it */
        
    res = yami_get_raw_cstring(body, body_size, &buffer_position,
        line, (size_t)len);
    assert(res == yami_ok);

    puts(line);
}

void run_server(const char * server_address)
{
    enum yami_result res;
    const size_t timeout = 1000; /* 1 second */
    
    struct yami_agent server_agent;

    /* create the server agent */
    /* and activate a listener on the given address */

    yami_init(&server_agent);
    
    /* message callback will be used to receive messages from client */
    
    yami_set_message_callback(&server_agent, &message_callback);

    res = yami_set_listener(&server_agent, server_address);
    assert(res == yami_ok);

    printf("The server is listening on %s\n", server_address);

    /* run the event loop */

    puts("waiting for messages...");

    while (1)
    {
        res = yami_do_some_work(&server_agent, timeout);
        assert(res == yami_ok || res == yami_timed_out);
    }

    /* never reaches here */
}

int main(int argc, char * argv[])
{
    char * server_address;
    
    if (argc != 2)
    {
        puts("expecting one parameter: server destination");
        return EXIT_FAILURE;
    }

    server_address = argv[1];

    run_server(server_address);
    
    /* unreachable */
    return 0;
}
