/* Copyright Inspirel Sp. z o.o. 2013-2015.
 * This file is part of YAMI4.
 *
 * YAMI4 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAMI4 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAMI4.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef YAMI4INDUSTRY_NETWORK_UTILS_H
#define YAMI4INDUSTRY_NETWORK_UTILS_H

#include "callbacks.h"
#include "channel.h"
#include "common.h"
#include "limits.h"
#include "options.h"

#include <stddef.h>
#include <stdint.h>

enum yami_result yami_create_listener(const char target[],
    const struct yami_options * options,
    int32_t * out_socket, enum yami_protocol * out_protocol,
    yami_io_error_function io_error_callback);
    
enum yami_result yami_clean_listener(int32_t sock,
    yami_io_error_function io_error_callback);

enum yami_result yami_create_outgoing_connection(const char target[],
    const struct yami_options * options,
    int32_t * out_socket, enum yami_protocol * out_protocol,
    uint32_t * out_host, uint16_t * out_port,
    yami_io_error_function io_error_callback);

enum yami_result yami_accept_tcp(int32_t listening_sock,
    const struct yami_options * options, char out_target[],
    int32_t * out_socket,
    yami_io_error_function io_error_callback);

enum yami_result yami_clean_connection(int32_t sock,
    yami_io_error_function io_error_callback);

enum yami_result yami_write_tcp(int32_t sock,
    const uint8_t data[], size_t from, size_t to, size_t * out_written,
    yami_io_error_function io_error_callback);

enum yami_result yami_send_udp(int32_t sock, uint32_t host, uint16_t port,
    const uint8_t data[], size_t size,
    yami_io_error_function io_error_callback);

enum yami_result yami_read_tcp(int32_t sock,
    uint8_t out_data[], size_t from, size_t to, size_t * out_readn,
    yami_io_error_function io_error_callback);

enum yami_result yami_receive_udp(const int32_t sock,
    char out_target[], int32_t format_target,
    uint8_t out_data[], const size_t size, size_t * const out_readn,
    const yami_io_error_function io_error_callback);

enum yami_result yami_wait_for_work(
    struct yami_channel channels[], size_t channels_array_size,
    int32_t listening_sock, int32_t * out_listening_sock_ready,
    size_t timeout,
    yami_io_error_function io_error_callback);

#endif /* YAMI4INDUSTRY_NETWORK_UTILS_H */
