/* Copyright Inspirel Sp. z o.o. 2013-2015.
 * This file is part of YAMI4.
 *
 * YAMI4 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAMI4 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAMI4.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "utils.h"

#include <stddef.h>
#include <stdint.h>

size_t yami_round_up_to_4(const size_t v)
{
    size_t res = v;
    const size_t rem = v % 4u;
    switch (rem)
    {
    case 1u:
        /* branch c1 */
        res += 3u;
        break;
    case 2u:
        /* branch c2 */
        res += 2u;
        break;
    case 3u:
        /* branch c3 */
        res += 1u;
        break;
    default:
        /* branch c4 */
        /* already at the 4 boundary */
        ;
    }
    
    return res;
}

size_t yami_round_up_to_8(const size_t v)
{
    size_t res = v;
    const size_t rem = v % 8u;
    switch (rem)
    {
    case 1u:
        /* branch c1 */
        res += 7u;
        break;
    case 2u:
        /* branch c2 */
        res += 6u;
        break;
    case 3u:
        /* branch c3 */
        res += 5u;
        break;
    case 4u:
        /* branch c4 */
        res += 4u;
        break;
    case 5u:
        /* branch c5 */
        res += 3u;
        break;
    case 6u:
        /* branch c6 */
        res += 2u;
        break;
    case 7u:
        /* branch c7 */
        res += 1u;
        break;
    default:
        /* branch c8 */
        /* already at the 8 boundary */
        ;
    }
    
    return res;
}

size_t yami_strlen(const char s[])
{
    size_t i = 0u;
    
    while ((int32_t)s[i] != (int32_t)'\0')
    {
        /* branch 1 */
        ++i;
    }
    
    return i;
}

int32_t yami_strcmp(const char s1[], const char s2[])
{
    int32_t res;
    size_t i = 0u;
    
    while (1)
    {
        /* branch 1 */
        const int32_t c1 = (int32_t)s1[i];
        const int32_t c2 = (int32_t)s2[i];
        
        /* decision strcmp_finished: (c1 != c2) || (c1 == (int32_t)'\0') */
        /*                        A: c1 != c2                            */
        /*                        B: c1 == (int32_t)'\0'                 */
        /* end                                                           */
        if ((c1 != c2) || (c1 == (int32_t)'\0'))
        {
            /* branch 1_c1 */
            res = c1 - c2;
            break;
        }
        
        ++i;
    }
    
    return res;
}

int32_t yami_strncmp(const char s1[], const char s2[], const size_t n)
{
    int32_t res = 0;
    size_t i = 0u;
    
    while (i < n)
    {
        /* branch 1 */
        const int32_t c1 = (int32_t)s1[i];
        const int32_t c2 = (int32_t)s2[i];
        
        if (c1 != c2)
        {
            /* branch 1_c1 */
            res = c1 - c2;
            break;
        }
        
        ++i;
    }
    
    return res;
}

void yami_strncpy(char dest[], const char src[], const size_t n)
{
    size_t i = 0u;
    
    while (i < n)
    {
        /* branch 1 */
        const char c = src[i];
        
        dest[i] = c;
        
        if ((int32_t)c == (int32_t)'\0')
        {
            /* branch 1_c1 */
            break;
        }
        
        ++i;
    }
}

int32_t yami_strfind(const char s[], const char c, const size_t from, const size_t to)
{
    size_t i = from;
    int32_t res = -1;
    
    while (i < to)
    {
        /* branch 1 */
        if ((int32_t)s[i] == (int32_t)c)
        {
            /* branch 1_c1 */
            res = (int32_t)i;
            break;
        }
        
        ++i;
    }
    
    return res;
}

void yami_memcpy(uint8_t dest[], const size_t dest_from,
    const uint8_t src[], const size_t src_from, const size_t n)
{
    size_t i;
    
    for (i = 0u; i != n; ++i)
    {
        /* branch 1 */
        dest[dest_from + i] = src[src_from + i];
    }
}

void yami_bzero(uint8_t dest[], const size_t n)
{
    size_t i;
    
    for (i = 0u; i != n; ++i)
    {
        /* branch 1 */
        dest[i] = (uint8_t)0u;
    }
}

int32_t yami_uint32_to_string(uint32_t v, char buf[], const size_t buf_from,
    const size_t buf_size)
{
    int32_t res;
    size_t index = buf_from;
    
    if (v == 0u)
    {
        /* branch c1 */
        buf[index] = (char)'0';
        res = (int32_t)index;
    }
    else
    {
        /* branch c2 */
        if (index < buf_size)
        {
            /* branch c2_c1 */
            while (v != 0u)
            {
                /* branch c2_c1_1 */
                uint32_t digit;
                uint32_t digit_code;
                
                digit = v % 10u;
                digit_code = digit + (uint32_t)'0';
                buf[index] = (char)digit_code;
                
                ++index;
                if (index == buf_size)
                {
                    /* branch c2_c1_1_c1 */
                    /* buffer is too small to format the given number */

                    res = -1;
                    
                    break;
                }

                v /= 10u;
            }
            
            if (v == 0u)
            {
                /* branch c2_c1_c2 */
                /* revert order of digits */
                size_t i = buf_from;
                size_t j = --index;
                while (i < j)
                {
                    /* branch c2_c1_c2_1 */
                    const char tmp = buf[i];
                    buf[i] = buf[j];
                    buf[j] = tmp;
                
                    ++i;
                    --j;
                }
            
                res = (int32_t)index;
            }
        }
        else
        {
            /* branch c2_c2 */
            /* function was called already at end-of-buffer */

            res = -1;
        }
    }
    
    return res;
}

int32_t yami_string_to_uint32(const char buf[],
    const size_t from, const size_t to, uint32_t * const out_value)
{
    int32_t res = 0; /* false */
    size_t index = from;
    uint32_t v = 0u;
    
    while (index < to)
    {
        /* branch 1 */
        int32_t c;
        
        c = (int32_t)buf[index];

        /* decision uint32_parse_invalid_char: (c < (int32_t)'0') || (c > (int32_t)'9') */
        /*                                  A: c < (int32_t)'0' */
        /*                                  B: c > (int32_t)'9' */
        /* end                                                  */
        if ((c < (int32_t)'0') || (c > (int32_t)'9'))
        {
            /* branch 1_c1 */
            res = 0; /* false */
            
            break;
        }
        
        v *= 10u;
        v += (uint32_t)c - (uint32_t)'0';
        
        res = 1; /* true */
        
        ++index;
    }
    
    if (res == 1)
    {
        /* branch c2 */
        *out_value = v;
    }
    
    return res;
}

int32_t yami_string_to_uint8(const char buf[],
    const size_t from, const size_t to, uint8_t * const out_value)
{
    uint32_t v;
    int32_t res = yami_string_to_uint32(buf, from, to, &v);
    
    if (res != 0)
    {
        /* branch c1 */
        if (v <= 255u)
        {
            /* branch c1_c1 */
            *out_value = (uint8_t)v;
        }
        else
        {
            /* branch c1_c2 */
            res = 0; /* false */
        }
    }
    
    return res;
}
