/* Copyright Inspirel Sp. z o.o. 2013-2015.
 * This file is part of YAMI4.
 *
 * YAMI4 is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * YAMI4 is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with YAMI4.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef YAMI4INDUSTRY_UTILS_H
#define YAMI4INDUSTRY_UTILS_H

#include <stddef.h>
#include <stdint.h>

#ifdef __cplusplus
extern "C"
{
#endif /* __cplusplus */

/** \brief Rounds the given value to the nearest 4 (for alignment).
 *
 * @param v value to round
 *
 * @return value rounded up to the nearset 4 boundary
 */
size_t yami_round_up_to_4(size_t v);

/** \brief Rounds the given value to the nearest 8 (for alignment).
 *
 * @param v value to round
 *
 * @return value rounded up to the nearset 8 boundary
 */
size_t yami_round_up_to_8(size_t v);

/** \brief Replacement for standard strlen.
 */
size_t yami_strlen(const char s[]);

/** \brief Replacement for standard strcmp.
 */
int32_t yami_strcmp(const char s1[], const char s2[]);

/** \brief Replacement for standard strncmp.
 */
int32_t yami_strncmp(const char s1[], const char s2[], size_t n);

/** \brief Replacement for standard strncpy.
 */
void yami_strncpy(char dest[], const char src[], size_t n);

/** \brief Finds the given character in nul-terminated string.
 *
 * @param s string to be searched in
 * @param c character to be searched for
 * @param from starting index in s
 * @param to last (not inclusive) index in s
 *
 * @return index or -1 if the given character is not found.
 */
int32_t yami_strfind(const char s[], char c, size_t from, size_t to);

/** \brief Replacement for standard memcpy.
 */
void yami_memcpy(uint8_t dest[], size_t dest_from,
    const uint8_t src[], size_t src_from, size_t n);

/** \brief Replacement for standard bzero.
 */
void yami_bzero(uint8_t dest[], size_t n);

/** \brief Converts uint32_t to string format.
 *
 * @param v value to be converted
 * @param buf buffer where the string will be formatted
 * @param buf_from starting index in buf
 * @param buf_size total size of buf
 *
 * @return index of last character in buf or -1 if buf_size is not big enough
 */
int32_t yami_uint32_to_string(uint32_t v, char buf[], size_t buf_from,
    size_t buf_size);

/** \brief Converts nul-terminated string to uint32_t.
 *
 * @param buf string to be converted
 * @param from starting index in buf
 * @param to last (not inclusive) index in buf
 * @param out_value value after conversion
 *
 * @return 1 on success or 0 if the given string cannot be converted to number
 */
int32_t yami_string_to_uint32(const char buf[],
    size_t from, size_t to, uint32_t * out_value);

/** \brief Converts nul-terminated string to uint8_t.
 *
 * @param buf string to be converted
 * @param from starting index in buf
 * @param to last (not inclusive) index in buf
 * @param out_value value after conversion
 *
 * @return 1 on success or 0 if the given string cannot be converted to number
 */
int32_t yami_string_to_uint8(const char buf[],
    size_t from, size_t to, uint8_t * out_value);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* YAMI4INDUSTRY_UTILS_H */
